library ieee;
use ieee.std_logic_1164.all;
library Unisim;
use Unisim.all;

entity my_input_fsl_ip is
  generic (
    C_FSL_DATA_SIZE   : integer := 8
    );
  port (
    FSL_Rst : in std_logic;
    FSL_Clk : in std_logic;		

    -- FSL master signals
    FSL0_M_Clk     : out std_logic;
    FSL0_M_Data    : out std_logic_vector(0 to C_FSL_DATA_SIZE-1);
    FSL0_M_Control : out std_logic;
    FSL0_M_Write   : out std_logic;
    FSL0_M_Full   : in std_logic;
    
    -- External ports
    FSL_INPUT_IP_GPIO	 : in std_logic_vector(0 to C_FSL_DATA_SIZE-1);
    IRQ 			 : out std_logic
	);
end entity my_input_fsl_ip;

architecture IMP of my_input_fsl_ip is
constant KEY : std_logic_vector(0 to 7) := "11110000";
    
signal FSL_INPUT_GPIO_reg : std_logic_vector(0 to C_FSL_DATA_SIZE-1);
signal data_changed, data_changed_a : std_logic; 
 
begin  -- architecture IMP

FSL0_M_Clk <= FSL_Clk;


process(FSL_Rst, FSL_Clk)
	begin
		if (FSL_Rst = '1') then
			FSL0_M_Write <= '0';
			FSL0_M_Control <= '0';
			FSL_INPUT_GPIO_reg <= (others => '0');
			FSL0_M_Data <= (others => '0');
			data_changed <= '0';
			data_changed_a <= '0';
		elsif FSL_Clk'event and FSL_Clk='1' then			
			if data_changed_a = '1' then
				FSL0_M_Write <= '0';
				data_changed_a <= '0';
				data_changed <= '0';
				IRQ <= '0';
			elsif data_changed = '1' then
				data_changed_a <= '1';
				--FSL0_M_Data <= FSL_INPUT_GPIO_reg;
				FSL0_M_Write <= '1';	
			end if;  
			--if (FSL_INPUT_GPIO_reg /= FSL_INPUT_IP_GPIO and FSL0_M_Full = '0') then
			if (FSL0_M_Full = '0') then
				data_changed <= '1';
				--FSL_INPUT_GPIO_reg <= FSL_INPUT_IP_GPIO;	
				--FSL0_M_Data <= FSL_INPUT_GPIO_reg;
				FSL0_M_Data <= FSL_INPUT_IP_GPIO;
				FSL0_M_Write <= '1';			
				if ( FSL_INPUT_IP_GPIO = KEY) then
					IRQ <= '1';
				end if;
			end if;
		end if;
	end process;

end architecture IMP;