-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- Flag_Handling_IMP.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        Flag_Handling_IMP.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              Flag_Handling_IMP.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2001-06-08    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library UNISIM;
use UNISIM.all;

architecture IMP of Flag_Handling is

  component LDCP is
    port (
      D   : in  std_logic;
      G   : in  std_logic;
      PRE : in  std_logic;
      CLR : in  std_logic;
      Q   : out std_logic);
  end component LDCP;

  component FDP is
    port (
      D   : in  std_logic;
      C   : in  std_logic;
      PRE : in  std_logic;
      Q   : out std_logic);
  end component FDP;

  signal direction       : std_logic;
  signal set_direction   : std_logic;
  signal clear_direction : std_logic;
  signal equal_counters  : std_logic;

  signal new_full  : std_logic;
  signal new_empty : std_logic;
  
begin  -- architecture IMP

  Direction_Latch : LDCP
    port map (
      D   => '0',                       -- [in  std_logic]
      G   => Reset,                     -- [in  std_logic]
      PRE => set_direction,             -- [in  std_logic]
      CLR => clear_direction,           -- [in  std_logic]
      Q   => direction);                -- [out std_logic]

  clear_direction <= (Write_Counter_Grey(C_SIZE-1) xor Read_Counter_Grey(C_SIZE-2)) and
                     (not (Write_Counter_Grey(C_SIZE-2) xor Read_Counter_Grey(C_SIZE-1)));

  set_direction <= (not (Write_Counter_Grey(C_SIZE-1) xor Read_Counter_Grey(C_SIZE-2))) and
                   (Write_Counter_Grey(C_SIZE-2) xor Read_Counter_Grey(C_SIZE-1));

  equal_counters <= '1' when Write_Counter_Grey = Read_Counter_Grey else '0';

  new_full  <= equal_counters and direction;
  new_empty <= equal_counters and not direction;

  FULL_DFF : FDP
    port map (
      D   => new_full,                  -- [in  std_logic]
      C   => Write_Clk,                 -- [in  std_logic]
      PRE => new_full,                  -- [in  std_logic]
      Q   => Full);                     -- [out std_logic]

  EMPTY_DFF : FDP
    port map (
      D   => new_empty,                 -- [in  std_logic]
      C   => Read_Clk,                  -- [in  std_logic]
      PRE => new_empty,                 -- [in  std_logic]
      Q   => Empty);                    -- [out std_logic]

end architecture IMP;



