-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- grey_counter_imp.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        grey_counter_imp.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              grey_counter_imp.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2001-06-08    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.numeric_std.all;

architecture IMP of Grey_Counter is

  signal bin_count     : std_logic_vector(C_SIZE-1 downto 0);
  signal new_bin_count : std_logic_vector(C_SIZE-1 downto 0);

  signal new_grey_count : std_logic_vector(C_SIZE-1 downto 0);
  
begin  -- architecture Grey_Counter

  new_bin_count <= std_logic_vector(unsigned(bin_count) + 1);

  Bin_Counter: process (Clk, Reset) is
  begin  -- process Bin_Counter
    if Reset = '1' then                 -- asynchronous reset (active high)
      bin_count <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (Count = '1') then
        bin_count <= new_bin_count;
      end if;
    end if;
  end process Bin_Counter;

  Bin_Value <= bin_count;
  
  Calc_New_Grey: process (new_bin_count) is
  begin  -- process Calc_New_Grey
    for I in C_SIZE-2 downto 0 loop
      new_grey_count(I) <= new_bin_count(I) xor new_bin_count(I+1);
    end loop;  -- I
    new_grey_count(C_SIZE-1) <= new_bin_count(C_SIZE-1);
  end process Calc_New_Grey;

  Grey_Counter: process (Clk, Reset) is
  begin  -- process Grey_Counter
    if Reset = '1' then                 -- asynchronous reset (active high)
      Grey_Value <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      Grey_Value <= new_grey_count;
    end if;
  end process Grey_Counter;
  
end architecture IMP;
